package com.cicdi.notify.wechat;


import com.cicdi.notify.AbstractNotifier;
import com.cicdi.notify.DefaultNotifyType;
import com.cicdi.notify.NotifyType;
import com.cicdi.notify.Provider;
import com.cicdi.notify.template.TemplateManager;
import org.springframework.http.MediaType;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.ClientResponse;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.util.UriComponentsBuilder;
import reactor.core.publisher.Mono;

import java.time.Duration;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

public class WeixinCorpNotifier extends AbstractNotifier<WechatMessageTemplate> {

    private final AtomicReference<String> accessToken = new AtomicReference<>();

    private long refreshTokenTime;

    private final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    private static final String tokenApi = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";

    private static final String notify = "https://qyapi.weixin.qq.com/cgi-bin/message/send";

    private final WechatCorpProperties properties;

    private final String notifierId;

    @Override
    public String getNotifierId() {
        return notifierId;
    }


    public WeixinCorpNotifier(String id, WechatCorpProperties properties, TemplateManager templateManager) {
        super(templateManager);
        this.properties = properties;
        this.notifierId = id;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.wechat;
    }

    @Override
    public Provider getProvider() {
        return WechatProvider.corpMessage;
    }

    @Override
    public void send(WechatMessageTemplate template, Map<String, Object> context) {
        String token1 = getToken();
        WebClient.create().post()
                .uri(UriComponentsBuilder.fromUriString(notify).queryParam("access_token", token1).toUriString())
                .contentType(MediaType.APPLICATION_JSON)
                .body(BodyInserters.fromValue(template.createJsonRequest(context)))
                .exchange()
                .flatMap(clientResponse -> clientResponse.bodyToMono(HashMap.class))
                .as(this::checkResult)
                .then();
    }

    private Mono<HashMap> checkResult(Mono<HashMap> msg) {
        return msg.doOnNext(map -> {
            String code = String.valueOf(map.get("errcode"));
            if ("0".equals(code)) {
//                log.info("发送微信企业通知成功");
            } else {
//                log.warn("发送微信企业通知失败:{}", map);
//                throw new BusinessException("发送微信企业通知失败:" + map.get("errmsg"), code);
            }
        });
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken.get() == null) {
            return requestToken();
        }
        return accessToken.get();
    }

    private String requestToken() {
        Mono<ClientResponse> responseMono = WebClient.create().get()
                .uri(UriComponentsBuilder.fromUriString(tokenApi)
                        .queryParam("corpId", properties.getCorpId())
                        .queryParam("corpSecret", properties.getCorpSecret())
                        .build().toUri())
                .exchange();

        responseMono.flatMap(resp -> {
            System.out.println(resp);
            return resp.bodyToMono(HashMap.class);
        })
                .map(map -> {
                    if (map.containsKey("access_token")) {
                        return map.get("access_token");
                    }
                    // TODO 自定义异常
//                    throw new BusinessException("获取Token失败:" + map.get("errmsg"), String.valueOf(map.get("errcode")));
                    return Mono.empty();
                })
                .cast(String.class)
                .doOnNext(r -> {
                    refreshTokenTime = System.currentTimeMillis();
                    accessToken.set(r);
                });
        return accessToken.get();
    }

    @Override
    public void close() {
        accessToken.set(null);
        refreshTokenTime = 0;
    }
}
